package org.jeecg.modules.basedata.controller;

import java.util.List;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.dto.basedata.ArchivesCostListRespDTO;
import org.jeecg.common.dto.basedata.ArchivesCostReqDTO;
import org.jeecg.modules.basedata.entity.BaseArchivesCost;
import org.jeecg.modules.basedata.entity.BaseArchivesMilestone;
import org.jeecg.modules.basedata.service.IBaseArchivesCostService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

 /**
   * @Author jihaosen
   * @date 2021/2/24
   */
@Api(value="费用档案接口")
@RestController
@RequestMapping("/basedata/baseArchivesCost")
@Slf4j
public class BaseArchivesCostController {
	 @Autowired
	 private IBaseArchivesCostService baseArchivesCostService;
	 @Autowired
	 private ISysSerialPatternService sysSerialPatternService;

	 @ApiOperation(value = "费用档案列表查询接口", notes = "费用档案列表查询")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
	 })
	 @GetMapping(value = "/list")
	 public Result<List<ArchivesCostListRespDTO>> queryPageList(String pkOrg) {
		 Result<List<ArchivesCostListRespDTO>> result = new Result<List<ArchivesCostListRespDTO>>();
		 QueryWrapper<BaseArchivesCost> queryWrapper = new QueryWrapper<>();
		 queryWrapper.eq("del_flag", CommonConstant.STATUS_NORMAL.toString());
		 if(StringUtils.isNotBlank(pkOrg)){
			 queryWrapper.eq("pk_org", pkOrg);
		 }
		 queryWrapper.orderByAsc("create_time");
		 List<BaseArchivesCost> list = baseArchivesCostService.list(queryWrapper);
		 List<ArchivesCostListRespDTO> archivesCostListRespDTOS = getArchivesCostData(list);
		 result.setSuccess(true);
		 result.setResult(archivesCostListRespDTOS);
		 return result;
	 }

	 @ApiOperation(value = "新增费用档案接口", notes = "新增费用档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="name", value="名称",required=true, dataType="String"),
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
	 })
	 @PostMapping(value = "/add")
	 @NotDuplicate
	 public Result<BaseArchivesCost> add(@RequestBody BaseArchivesCost baseArchivesCost, BindingResult bindingResult) {
		 Result<BaseArchivesCost> result = new Result<BaseArchivesCost>();
		 try {
			 StringBuilder sb = new StringBuilder();
			 if (bindingResult.hasErrors()){
				 //记录错误信息
				 bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				 result.error500(sb.toString());
				 return result;
			 }
			 // 设置编码
			 CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(baseArchivesCost, "base_archives_cost", "code", true);
			 if(!nextSerial.isSucceed()){
				 throw new RuntimeException("获取编号失败");
			 }
			 baseArchivesCost.setCode(nextSerial.getContent());
			 baseArchivesCost.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
			 baseArchivesCostService.save(baseArchivesCost);
			 result.success("添加成功！");
		 } catch (Exception e) {
			 e.printStackTrace();
			 log.info(e.getMessage());
			 result.error500("操作失败");
		 }
		 return result;
	 }

	 @ApiOperation(value = "编辑费用档案接口", notes = "编辑费用档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
			 @ApiImplicitParam(name="name", value="名称",required=true, dataType="String"),
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
	 })
	 @PostMapping(value = "/edit")
	 @NotDuplicate
	 public Result<BaseArchivesCost> edit(@RequestBody BaseArchivesCost baseArchivesCost) {
		 Result<BaseArchivesCost> result = new Result<BaseArchivesCost>();
		 BaseArchivesCost baseArchivesCostEntity = baseArchivesCostService.getById(baseArchivesCost.getId());
		 if(baseArchivesCostEntity == null) {
			 result.error500("未找到对应实体");
		 } else {
//			 // 设置编码
//			 CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(baseArchivesCost, "base_archives_cost", "code", true);
//			 if(!nextSerial.isSucceed()){
//				 throw new RuntimeException("获取编号失败");
//			 }
			 boolean ok = baseArchivesCostService.updateById(baseArchivesCost);
			 if(ok) {
				 result.success("修改成功!");
			 }
		 }
		 return result;
	 }

	 @ApiOperation(value = "删除费用档案接口", notes = "删除费用档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
	 })
	 @PostMapping(value = "/delete")
	 @NotDuplicate
	 public Result<BaseArchivesCost> delete(@RequestBody ArchivesCostReqDTO reqDTO) {
		 Result<BaseArchivesCost> result = new Result<BaseArchivesCost>();
		 BaseArchivesCost baseArchivesCost = baseArchivesCostService.getById(reqDTO.getId());
		 if(baseArchivesCost==null) {
			 result.error500("未找到对应实体");
		 }else {
			 baseArchivesCost.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
			 boolean ok = baseArchivesCostService.updateById(baseArchivesCost);
			 if(ok) {
				 result.success("删除成功!");
			 }
		 }
		 return result;
	 }

	 @ApiOperation(value = "根据id查询费用档案接口", notes = "根据id查询费用档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
	 })
	 @GetMapping(value = "/queryById")
	 public Result<BaseArchivesCost> queryById(@RequestParam(name = "id", required = true) String id) {
		 Result<BaseArchivesCost> result = new Result<BaseArchivesCost>();
		 BaseArchivesCost baseArchivesCost = baseArchivesCostService.getById(id);
		 if(baseArchivesCost == null) {
			 result.error500("未找到对应实体");
		 } else {
			 result.setResult(baseArchivesCost);
			 result.setSuccess(true);
		 }
		 return result;
	 }




	 /**
	  * @dsec 获取费用档案数据
	  */
	 private List<ArchivesCostListRespDTO> getArchivesCostData(List<BaseArchivesCost> list) {
	 	List<ArchivesCostListRespDTO> archivesCostListRespDTOS = Lists.newArrayList();
	 	for(BaseArchivesCost baseArchivesCost : list) {
			archivesCostListRespDTOS.add(new ArchivesCostListRespDTO(baseArchivesCost));
		}
	 	return archivesCostListRespDTOS;
	 }
 }